@@ -1,21 +1,30 @@
 import base64
 import os
-import thread
-import urllib
+try:
+    from _thread import get_ident
+except ImportError:
+    from thread import get_ident
+try:
+    from urllib import quote
+except ImportError:
+    from urllib.parse import quote
+from sqlobject import col
+from sqlobject import dberrors
 from sqlobject.dbconnection import DBAPI, Boolean
-from sqlobject import col, sqlbuilder
-from sqlobject.dberrors import *
+
 
 sqlite2_Binary = None
 
+
 class ErrorMessage(str):
     def __new__(cls, e):
-        obj = str.__new__(cls, e[0])
+        obj = str.__new__(cls, e.args[0])
         obj.code = None
         obj.module = e.__module__
         obj.exception = e.__class__.__name__
         return obj
 
+
 class SQLiteConnection(DBAPI):
 
     supportTransactions = True
@@ -39,13 +48,16 @@ class SQLiteConnection(DBAPI):
                         import sqlite
                         self.using_sqlite2 = False
                 else:
-                    raise ValueError('Unknown SQLite driver "%s", expected pysqlite2, sqlite3 or sqlite' % driver)
+                    raise ValueError(
+                        'Unknown SQLite driver "%s", '
+                        'expected pysqlite2, sqlite3 or sqlite' % driver)
             except ImportError:
                 pass
             else:
                 break
         else:
-            raise ImportError('Cannot find an SQLite driver, tried %s' % drivers)
+            raise ImportError(
+                'Cannot find an SQLite driver, tried %s' % drivers)
         if self.using_sqlite2:
             sqlite.encode = base64.encodestring
             sqlite.decode = base64.decodestring
@@ -121,7 +133,7 @@ class SQLiteConnection(DBAPI):
                 path = "//" + path
             else:
                 path = "///" + path
-            path = urllib.quote(path)
+            path = quote(path)
         return 'sqlite:%s' % path
 
     def getConnection(self):
@@ -133,9 +145,8 @@ class SQLiteConnection(DBAPI):
             self._connectionNumbers[id(conn)] = self._connectionCount
             self._connectionCount += 1
             return conn
-        threadid = thread.get_ident()
-        if (self._pool is not None
-            and threadid in self._threadPool):
+        threadid = get_ident()
+        if (self._pool is not None and threadid in self._threadPool):
             conn = self._threadPool[threadid]
             del self._threadPool[threadid]
             if conn in self._pool:
@@ -149,7 +160,8 @@ class SQLiteConnection(DBAPI):
         if self.debug:
             s = 'ACQUIRE'
             if self._pool is not None:
-                s += ' pool=[%s]' % ', '.join([str(self._connectionNumbers[id(v)]) for v in self._pool])
+                s += ' pool=[%s]' % ', '.join(
+                    [str(self._connectionNumbers[id(v)]) for v in self._pool])
             self.printDebug(conn, s, 'Pool')
         return conn
 
@@ -158,8 +170,8 @@ class SQLiteConnection(DBAPI):
             return
         threadid = self._threadOrigination.get(id(conn))
         DBAPI.releaseConnection(self, conn, explicit=explicit)
-        if (self._pool is not None and threadid
-            and threadid not in self._threadPool):
+        if (self._pool is not None and threadid and
+                threadid not in self._threadPool):
             self._threadPool[threadid] = conn
         else:
             if self._pool and conn in self._pool:
@@ -191,7 +203,7 @@ class SQLiteConnection(DBAPI):
         if self._memory:
             return self._memoryConn
         conn = self.module.connect(self.filename, **self._connOptions)
-        conn.text_factory = str # Convert text data to str, not unicode
+        conn.text_factory = str  # Convert text data to str, not unicode
         return conn
 
     def close(self):
@@ -206,31 +218,31 @@ class SQLiteConnection(DBAPI):
             self.printDebug(conn, query, 'QueryR')
         try:
             return cursor.execute(query)
-        except self.module.OperationalError, e:
-            raise OperationalError(ErrorMessage(e))
-        except self.module.IntegrityError, e:
+        except self.module.OperationalError as e:
+            raise dberrors.OperationalError(ErrorMessage(e))
+        except self.module.IntegrityError as e:
             msg = ErrorMessage(e)
             if msg.startswith('column') and msg.endswith('not unique') \
-            or msg.startswith('UNIQUE constraint failed:'):
-                raise DuplicateEntryError(msg)
+                    or msg.startswith('UNIQUE constraint failed:'):
+                raise dberrors.DuplicateEntryError(msg)
             else:
-                raise IntegrityError(msg)
-        except self.module.InternalError, e:
-            raise InternalError(ErrorMessage(e))
-        except self.module.ProgrammingError, e:
-            raise ProgrammingError(ErrorMessage(e))
-        except self.module.DataError, e:
-            raise DataError(ErrorMessage(e))
-        except self.module.NotSupportedError, e:
-            raise NotSupportedError(ErrorMessage(e))
-        except self.module.DatabaseError, e:
-            raise DatabaseError(ErrorMessage(e))
-        except self.module.InterfaceError, e:
-            raise InterfaceError(ErrorMessage(e))
-        except self.module.Warning, e:
+                raise dberrors.IntegrityError(msg)
+        except self.module.InternalError as e:
+            raise dberrors.InternalError(ErrorMessage(e))
+        except self.module.ProgrammingError as e:
+            raise dberrors.ProgrammingError(ErrorMessage(e))
+        except self.module.DataError as e:
+            raise dberrors.DataError(ErrorMessage(e))
+        except self.module.NotSupportedError as e:
+            raise dberrors.NotSupportedError(ErrorMessage(e))
+        except self.module.DatabaseError as e:
+            raise dberrors.DatabaseError(ErrorMessage(e))
+        except self.module.InterfaceError as e:
+            raise dberrors.InterfaceError(ErrorMessage(e))
+        except self.module.Warning as e:
             raise Warning(ErrorMessage(e))
-        except self.module.Error, e:
-            raise Error(ErrorMessage(e))
+        except self.module.Error as e:
+            raise dberrors.Error(ErrorMessage(e))
 
     def _queryInsertID(self, conn, soInstance, id, names, values):
         table = soInstance.sqlmeta.table
@@ -265,7 +277,7 @@ class SQLiteConnection(DBAPI):
             return "%s LIMIT %i" % (query, end)
         if not end:
             return "%s LIMIT 0 OFFSET %i" % (query, start)
-        return "%s LIMIT %i OFFSET %i" % (query, end-start, start)
+        return "%s LIMIT %i OFFSET %i" % (query, end - start, start)
 
     def createColumn(self, soClass, col):
         return col.sqliteCreateSQL()
@@ -285,7 +297,9 @@ class SQLiteConnection(DBAPI):
         return 'INT NOT NULL'
 
     def tableExists(self, tableName):
-        result = self.queryOne("SELECT tbl_name FROM sqlite_master WHERE type='table' AND tbl_name = '%s'" % tableName)
+        result = self.queryOne(
+            "SELECT tbl_name FROM sqlite_master "
+            "WHERE type='table' AND tbl_name = '%s'" % tableName)
         # turn it into a boolean:
         return not not result
 
@@ -296,7 +310,7 @@ class SQLiteConnection(DBAPI):
         self.query('ALTER TABLE %s ADD COLUMN %s' %
                    (tableName,
                     column.sqliteCreateSQL()))
-        self.query('VACUUM %s' % tableName)
+        self.query('VACUUM')
 
     def delColumn(self, sqlmeta, column):
         self.recreateTableWithoutColumn(sqlmeta, column)
@@ -304,12 +318,13 @@ class SQLiteConnection(DBAPI):
     def recreateTableWithoutColumn(self, sqlmeta, column):
         new_name = sqlmeta.table + '_ORIGINAL'
         self.query('ALTER TABLE %s RENAME TO %s' % (sqlmeta.table, new_name))
-        cols = [self._createIDColumn(sqlmeta)] \
-                     + [self.createColumn(None, col)
-                        for col in sqlmeta.columnList if col.name != column.name]
+        cols = [self._createIDColumn(sqlmeta)] + \
+            [self.createColumn(None, col)
+                for col in sqlmeta.columnList if col.name != column.name]
         cols = ",\n".join(["    %s" % c for c in cols])
         self.query('CREATE TABLE %s (\n%s\n)' % (sqlmeta.table, cols))
-        all_columns = ', '.join([sqlmeta.idName] + [col.dbName for col in sqlmeta.columnList])
+        all_columns = ', '.join(
+            [sqlmeta.idName] + [col.dbName for col in sqlmeta.columnList])
         self.query('INSERT INTO %s (%s) SELECT %s FROM %s' % (
             sqlmeta.table, all_columns, all_columns, new_name))
         self.query('DROP TABLE %s' % new_name)
@@ -340,17 +355,22 @@ class SQLiteConnection(DBAPI):
         return results
 
     def _columnsFromSchemaParse(self, tableName, soClass):
-        colData = self.queryOne("SELECT sql FROM sqlite_master WHERE type='table' AND name='%s'"
-                                % tableName)
+        colData = self.queryOne(
+            "SELECT sql FROM sqlite_master WHERE type='table' AND name='%s'" %
+            tableName)
         if not colData:
-            raise ValueError('The table %s was not found in the database. Load failed.' % tableName)
+            raise ValueError(
+                'The table %s was not found in the database. Load failed.' %
+                tableName)
         colData = colData[0].split('(', 1)[1].strip()[:-2]
         while True:
             start = colData.find('(')
-            if start == -1: break
+            if start == -1:
+                break
             end = colData.find(')', start)
-            if end == -1: break
-            colData = colData[:start] + colData[end+1:]
+            if end == -1:
+                break
+            colData = colData[:start] + colData[end + 1:]
         results = []
         for colDesc in colData.split(','):
             parts = colDesc.strip().split(' ', 2)
@@ -372,7 +392,9 @@ class SQLiteConnection(DBAPI):
             kw['dbName'] = field
             import re
             nullble = re.search(r'(\b\S*)\sNULL', index_info)
-            default = re.search(r"DEFAULT\s((?:\d[\dA-FX.]*)|(?:'[^']*')|(?:#[^#]*#))", index_info)
+            default = re.search(
+                r"DEFAULT\s((?:\d[\dA-FX.]*)|(?:'[^']*')|(?:#[^#]*#))",
+                index_info)
             kw['notNone'] = nullble and nullble.group(1) == 'NOT'
             kw['default'] = default and default.group(1)
             # @@ skip key...
@@ -385,9 +407,9 @@ class SQLiteConnection(DBAPI):
         if t.find('INT') >= 0:
             return col.IntCol, {}
         elif t.find('TEXT') >= 0 or t.find('CHAR') >= 0 or t.find('CLOB') >= 0:
-            return col.StringCol, {'length': 2**32-1}
+            return col.StringCol, {'length': 2 ** 32 - 1}
         elif t.find('BLOB') >= 0:
-            return col.BLOBCol, {"length": 2**32-1}
+            return col.BLOBCol, {"length": 2 ** 32 - 1}
         elif t.find('REAL') >= 0 or t.find('FLOAT') >= 0:
             return col.FloatCol, {}
         elif t.find('DECIMAL') >= 0:
