@@ -1,24 +1,19 @@
 """
 The framework for making database tests.
 """
+from __future__ import print_function
 
-import sys
+import logging
 import os
-import re
-from py.test import raises
-import py
+import sys
+from pytest import raises, skip
 import sqlobject
+from sqlobject.col import use_microseconds
 import sqlobject.conftest as conftest
 
-try:
-    import logging
-    loggingModuleAvailable = True
-except ImportError:
-    loggingModuleAvailable = False
-
 if sys.platform[:3] == "win":
     def getcwd():
-        return os.getcwd().replace(':', '|')
+        return os.getcwd().replace('\\', '/')
 else:
     getcwd = os.getcwd
 
@@ -35,19 +30,21 @@ and you can use it like::
 
     def test_featureX():
         if not supports('featureX'):
-            return
+            pytest.skip("Doesn't support featureX")
 """
 supportsMatrix = {
-    '+exceptions': 'mysql sqlite',
-    '-transactions': 'mysql rdbhost',
-    '-dropTableCascade': 'sybase mssql',
-    '-expressionIndex': 'mysql sqlite firebird mssql',
     '-blobData': 'mssql rdbhost',
     '-decimalColumn': 'mssql',
+    '-dropTableCascade': 'sybase mssql mysql',
     '-emptyTable': 'mssql',
-    '-limitSelect' : 'mssql',
-    '+schema' : 'postgres',
-    }
+    '+exceptions': 'mysql postgres sqlite',
+    '-expressionIndex': 'mysql sqlite firebird mssql',
+    '-limitSelect': 'mssql',
+    '+memorydb': 'sqlite',
+    '+rlike': 'mysql postgres sqlite',
+    '+schema': 'postgres',
+    '-transactions': 'mysql rdbhost',
+}
 
 
 def setupClass(soClasses, force=False):
@@ -71,22 +68,31 @@ def setupClass(soClasses, force=False):
         soClasses = [soClasses]
     connection = getConnection()
     for soClass in soClasses:
-        ## This would be an alternate way to register connections...
-        #try:
+        # This would be an alternate way to register connections:
+        # try:
         #    hub
-        #except NameError:
+        # except NameError:
         #    hub = sqlobject.dbconnection.ConnectionHub()
-        #soClass._connection = hub
-        #hub.threadConnection = connection
-        #hub.processConnection = connection
+        # soClass._connection = hub
+        # hub.threadConnection = connection
+        # hub.processConnection = connection
         soClass._connection = connection
     installOrClear(soClasses, force=force)
     return soClasses
 
+
+def speedupSQLiteConnection(connection):
+    connection.query("PRAGMA synchronous=OFF")
+    connection.query("PRAGMA count_changes=OFF")
+    connection.query("PRAGMA journal_mode=MEMORY")
+    connection.query("PRAGMA temp_store=MEMORY")
+
 installedDBFilename = os.path.join(getcwd(), 'dbs_data.tmp')
 
 installedDBTracker = sqlobject.connectionForURI(
     'sqlite:///' + installedDBFilename)
+speedupSQLiteConnection(installedDBTracker)
+
 
 def getConnection(**kw):
     name = getConnectionURI()
@@ -95,8 +101,11 @@ def getConnection(**kw):
         conn.debug = True
     if conftest.option.show_sql_output:
         conn.debugOutput = True
+    if (conn.dbName == 'sqlite') and not conn._memory:
+        speedupSQLiteConnection(conn)
     return conn
 
+
 def getConnectionURI():
     name = conftest.option.Database
     if name in conftest.connectionShortcuts:
@@ -105,10 +114,18 @@ def getConnectionURI():
 
 try:
     connection = getConnection()
-except Exception, e:
+except Exception as e:
     # At least this module should be importable...
-    print >> sys.stderr, (
-        "Could not open database: %s" % e)
+    # The module was imported during documentation building
+    if 'sphinx' not in sys.modules:
+        print("Could not open database: %s" % e, file=sys.stderr)
+else:
+    if (connection.dbName == 'firebird') \
+        or (
+            (connection.dbName == 'mysql') and
+            (os.environ.get('APPVEYOR'))
+    ):
+        use_microseconds(False)
 
 
 class InstalledTestDatabase(sqlobject.SQLObject):
@@ -212,6 +229,7 @@ class InstalledTestDatabase(sqlobject.SQLObject):
 
 installOrClear = InstalledTestDatabase.installOrClear
 
+
 class Dummy(object):
 
     """
@@ -222,12 +240,6 @@ class Dummy(object):
         for name, value in kw.items():
             setattr(self, name, value)
 
-def d(**kw):
-    """
-    Because ``dict(**kw)`` doesn't work in Python 2.2, this is a
-    replacement.
-    """
-    return kw
 
 def inserts(cls, data, schema=None):
     """
@@ -260,6 +272,7 @@ def inserts(cls, data, schema=None):
         results.append(cls(**args))
     return results
 
+
 def supports(feature):
     dbName = connection.dbName
     support = supportsMatrix.get('+' + feature, None)
@@ -276,10 +289,11 @@ def supports(feature):
         "The supportMatrix does not list this feature: %r"
         % feature)
 
-    
+
 # To avoid name clashes:
 _inserts = inserts
 
+
 def setSQLiteConnectionFactory(TableClass, factory):
     from sqlobject.sqlite.sqliteconnection import SQLiteConnection
     conn = TableClass._connection
@@ -290,12 +304,15 @@ def setSQLiteConnectionFactory(TableClass, factory):
         debugThreading=conn.debugThreading, registry=conn.registry,
         factory=factory
     )
+    speedupSQLiteConnection(TableClass._connection)
     installOrClear([TableClass])
 
+
 def deprecated_module():
     sqlobject.main.warnings_level = None
     sqlobject.main.exception_level = None
 
+
 def setup_module(mod):
     # modules with '_old' test backward compatible methods, so they
     # don't get warnings or errors.
@@ -309,13 +326,13 @@ def setup_module(mod):
         sqlobject.main.warnings_level = None
         sqlobject.main.exception_level = 0
 
+
 def teardown_module(mod=None):
     sqlobject.main.warnings_level = None
     sqlobject.main.exception_level = 0
 
+
 def setupLogging():
-    if not loggingModuleAvailable:
-        return
     fmt = '[%(asctime)s] %(name)s %(levelname)s: %(message)s'
     formatter = logging.Formatter(fmt)
     hdlr = logging.StreamHandler(sys.stderr)
@@ -324,6 +341,24 @@ def setupLogging():
     logger = logging.getLogger()
     logger.addHandler(hdlr)
 
-__all__ = ['getConnection', 'getConnectionURI', 'setupClass', 'Dummy', 'raises',
-           'd', 'inserts', 'supports', 'deprecated_module',
-           'setup_module', 'teardown_module', 'setupLogging']
+
+def setupCyclicClasses(*classes):
+    if not supports('dropTableCascade'):
+        skip("dropTableCascade isn't supported")
+    conn = getConnection()
+    for soClass in classes:
+        soClass.setConnection(conn)
+        soClass.dropTable(ifExists=True, cascade=True)
+
+    constraints = []
+    for soClass in classes:
+        constraints += soClass.createTable(ifNotExists=True,
+                                           applyConstraints=False)
+    for constraint in constraints:
+        conn.query(constraint)
+
+
+__all__ = ['Dummy', 'deprecated_module', 'getConnection', 'getConnectionURI',
+           'inserts', 'raises', 'setupClass', 'setupCyclicClasses',
+           'setupLogging', 'setup_module', 'supports', 'teardown_module',
+           ]
